@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 
 import curses
+from exam_terminal import utlity
 import logging
 import os
 import sys
@@ -15,11 +16,25 @@ from typing import Tuple, Dict
 import yaml
 from fpdf import FPDF
 
+from . import utlity
+
 logger = logging.getLogger()
 
 
 class ExamTerminal:
+    """
+    This class defines the exam terminal and its function.
+    """
+
     def __init__(self, exam_filepath: str) -> None:
+        """
+        Object constructor method
+
+        Parameters:
+            exam_filepath (str): The path to the exam file
+        Returns: 
+            None
+        """
         # Loading exam contents
         self.exam_contents = self.__load_parse_examfile(exam_filepath)
         if not self.exam_contents:
@@ -62,6 +77,14 @@ class ExamTerminal:
     ###############################################################################################
 
     def __load_parse_examfile(self, filepath: str) -> Dict:
+        """
+        Loading and pasing a exam file form specified path
+
+        Parameters:
+            filepath (str): The path to the exam file
+        Returns: 
+            return (Dict): Loaded and parsed info of exam file contents
+        """
         # Load the examp file
         logger.debug(f"Loading specified exam file: '{filepath}' ...")
         try:
@@ -127,11 +150,20 @@ class ExamTerminal:
         return self.exam_contents
 
     def __basic_screen_setup(self, scr, halfdelay: bool) -> None:
+        """
+        Basic configurations of the current curses terminal screen
+
+        Parameters:
+            scr (obj)      : Handle for curses terminal screen handle
+            halfdely (bool): If True, refresh specified 1/10th of second, else refresh 25.5 seconds
+        Returns: 
+            None
+        """
         # Hiding the cursor
         curses.curs_set(0)
 
         # Load curses colors
-        self.color, self.decor = self.__load_curses_colors_decor()
+        self.color, self.decor = utlity.load_curses_colors_decor()
 
         # Turn off echo
         curses.noecho()
@@ -143,6 +175,14 @@ class ExamTerminal:
             curses.halfdelay(255)
 
     def __check_terminal_size(self, scr) -> None:
+        """
+        Checking if current terminal size is sufficient, if it is not, display warning.
+
+        Parameters:
+            scr (obj): Handle for curses terminal screen handle
+        Returns: 
+            None
+        """
         # Getting the screen height and width
         term_height, term_width = scr.getmaxyx()
         
@@ -154,7 +194,7 @@ class ExamTerminal:
             scr.addstr(1, 1, f"[Terminal Size: W:{term_width}, H:{term_height}]", self.color['grey-light'])
 
         k = 0
-        KEYS = self.__load_keys()
+        KEYS = utlity.load_keys()
         while not self.terminal_size_good:
             if k in KEYS['QUIT']:
                 self.exam_exit = True
@@ -175,34 +215,26 @@ class ExamTerminal:
                 ''
                 'To quit program press "Q" or "ESC"'
                 ]
-            # self.__draw_message_box(scr, message_lines)
+            # utlity.draw_message_box(scr, message_lines)
             for y, line in enumerate(message_lines):
                 scr.addstr(1 + y, 1, line, self.decor['bold'])
 
             scr.refresh()
             k = scr.getch()
 
+    ###############################################################################################
 
-    def __draw_screen_border(self, scr, color:list) -> None:
-        scr.attron(color)
-        scr.border(0)
-        scr.attroff(color)
+    def __draw_selection_menu(self, scr, selections:list, start_y:int) -> None:
+        """
+        Draw a Selection menu at the bottom of the screen with a specified options
 
-    def __draw_horizontal_seperator(self, scr, y:int, color:list) -> None:
-        # Getting the screen height and width
-        term_height, term_width = scr.getmaxyx()
-
-        if y < term_height - 2 and y > 1:
-            for x in range(term_width - 2):
-                scr.addstr(y, x + 1, '-', color)
-
-    def __draw_vertical_seperator(self, scr, x:int, color:list) -> None:
-        # Getting the screen height and width
-        term_height, term_width = scr.getmaxyx()
-
-        # TODO
-
-    def __draw_selection_menu(self, scr, selections:list, start_y:int):
+        Parameters:
+            scr (obj)         : Handle for curses terminal screen handle
+            selections (list) : Menu selection for each list item
+            start_y (int)     : Line/row number at which selection menu starts
+        Returns: 
+            None
+        """
         # Getting the terminal size
         _, term_width = scr.getmaxyx()  
 
@@ -227,11 +259,20 @@ class ExamTerminal:
                 color = self.color['grey-light']
             scr.addstr(y + s + 1 + start_y, term_width // 2 - len(selection) // 2, selection, color)
 
-
     def __draw_message_box(self, scr, message_lines: list) -> None:
+        """
+        Draw a message box in the middle of the terminal screen
+
+        Parameters:
+            scr (obj)            : Handle for curses terminal screen handle
+            message_lines (list) : Lines of text in each list item
+        Returns: 
+            None
+        """
         term_height, term_width = scr.getmaxyx()
 
-        height, width, y, x = self.__get_message_box_size(term_height, term_width, message_lines)
+        # Getting the message box size and position and creating the message box
+        height, width, y, x = utlity.get_message_box_size(term_height, term_width, message_lines)
         message_box = curses.newwin(height, width, y, x)
         message_box.box()
         message_box.border()
@@ -245,34 +286,22 @@ class ExamTerminal:
         # Refresh the messgae box
         message_box.refresh()
 
-
-    def __get_message_box_size(self, term_height:int, term_width:int, message_lines:list) -> Tuple[int, int, int, int]:
-        # Create a box (Height, Width, y, x) (Positions are top left)
-        box_height = len(message_lines) + 4
-        box_width = int(term_width / 1.5)  # Alternative: len(max(message_lines, key=len)) + 12
-        box_y = int(term_height / 2 - box_height / 2)
-        box_x = int(term_width / 2 - box_width / 2)
-        
-        return box_height, box_width, box_y, box_x
-
-    def __get_progress_bar(self, exam_progress, bar_char_width=60, bar_char_full='|', bar_char_empty='-') -> str:
-        progress_str = []
-        for i in range(bar_char_width):
-            # TODO: Different colors for different parts of the progress bar somehow
-            if i <= exam_progress * bar_char_width:
-                progress_str.append(bar_char_full)
-            else:
-                progress_str.append(bar_char_empty)
-
-        progress_str = "".join(progress_str)
-        return progress_str
-
     ###############################################################################################
 
     def draw_menu(self, scr) -> Tuple[str, bool]:
+        """
+        Draw a the main menu on the screen
+
+        Parameters:
+            scr (obj)         : Handle for curses terminal screen handle
+        Returns: 
+            menu option (str)  : Selection menu option user selected (ie. quit)
+            successfull (bool) : True if no error, else False
+        """
+
         # Setting up basic stuff for curses and load keys
         self.__basic_screen_setup(scr, halfdelay=False)
-        KEYS = self.__load_keys()
+        KEYS = utlity.load_keys()
 
         # User key input (ASCII)
         k = 0
@@ -317,10 +346,10 @@ class ExamTerminal:
             self.__check_terminal_size(scr)
 
             # Drawing the screen border
-            self.__draw_screen_border(scr, self.color['grey-dark'])
+            utlity.draw_screen_border(scr, self.color['grey-dark'])
 
             # Show software name/title
-            scr.addstr(term_height - 2, 2, self.__load_software_ascii_name(), self.color['grey-dark'])
+            scr.addstr(term_height - 2, 2, utlity.load_software_ascii_name(), self.color['grey-dark'])
 
             ########################################################################################
 
@@ -330,15 +359,15 @@ class ExamTerminal:
             start_x = [5, 22]
 
             line = f"{self.exam_contents['exam']['exam_title']}"
-            scr.addstr(start_y, self.__center_x(term_width, line), line, self.decor['bold'])
+            scr.addstr(start_y, utlity.center_x(term_width, line), line, self.decor['bold'])
             start_y += 1
 
             line = f"{self.exam_contents['exam']['exam_author']}"
-            scr.addstr(start_y, self.__center_x(term_width, line), line, self.color['grey-light'])
+            scr.addstr(start_y, utlity.center_x(term_width, line), line, self.color['grey-light'])
             start_y += 1
 
             line = f"{self.exam_contents['exam']['exam_edit_date']}"
-            scr.addstr(start_y, self.__center_x(term_width, line), line, self.color['grey-light'])
+            scr.addstr(start_y, utlity.center_x(term_width, line), line, self.color['grey-light'])
             start_y += 3
 
 
@@ -356,7 +385,7 @@ class ExamTerminal:
                 scr.addstr(start_y , x, line, self.color['default'])
             start_y += 2
 
-            lines = ["Questions:", f"52"]
+            lines = ["Questions:", f"{self.exam_contents['exam']['exam_questions_count']}"]
             for x, line in zip(start_x, lines):
                 scr.addstr(start_y, x, line, self.color['default'])
             start_y += 2
@@ -374,7 +403,7 @@ class ExamTerminal:
             ########################################################################################
 
             selections = ["Begin Exam", "Quit"]
-            self.__draw_horizontal_seperator(scr, term_height - len(selections) - 4, self.color['grey-dark'])
+            utlity.draw_horizontal_seperator(scr, term_height - len(selections) - 4, self.color['grey-dark'])
             start_y = term_height - len(selections) - 7
             self.__draw_selection_menu(scr, selections, start_y)
 
@@ -406,12 +435,30 @@ class ExamTerminal:
             k = scr.getch()
 
     def show_menu(self) -> Tuple[str, bool]:
+        """
+        Curses wrapper function for drawing main menu on screen
+
+        Parameters:
+            None
+        Returns: 
+            menu option (str)  : Selection menu option user selected (ie. quit)
+            successfull (bool) : True if no error, else False
+        """
+
         return curses.wrapper(self.draw_menu)
 
     ###############################################################################################
 
     def __exam_timer_thread(self) -> None:
+        """
+        Exam timer that keeps track of elapsed exam time and exam paused time.
+        Interactions to the main thread are via object properties.
 
+        Parameters:
+            None
+        Returns: 
+            None
+        """
         logger.debug('Starting exam timer thread ...')
         while self.is_timer_timing:
             self.global_elapsed_time = time() - self.exam_begin_time 
@@ -426,13 +473,12 @@ class ExamTerminal:
             else:
                 # Time Spend paused
                 self.exam_paused_elapsed_time = self.global_elapsed_time - self.exam_elapsed_time
-
         logger.debug('Exam timer thread ended')
 
     def draw_question(self, scr, question:dict) -> Tuple[str, bool]:
         # Setting up basic stuff for curses and load keys
         self.__basic_screen_setup(scr, halfdelay=True)
-        KEYS = self.__load_keys()
+        KEYS = utlity.load_keys()
 
         start_y = 3
         question_x = 4
@@ -487,7 +533,7 @@ class ExamTerminal:
             self.__check_terminal_size(scr)
 
             # Drawing the screen border
-            self.__draw_screen_border(scr, self.color['grey-dark'])
+            utlity.draw_screen_border(scr, self.color['grey-dark'])
             
             ########################################################################################
 
@@ -507,7 +553,7 @@ class ExamTerminal:
             for l, line in enumerate(question_wrap):
                 scr.addstr(start_y + l - 1, question_x, line, self.color['default'] | self.decor['bold'])
 
-            self.__draw_horizontal_seperator(scr, len(question_wrap) + 3, self.color['grey-dark'])
+            utlity.draw_horizontal_seperator(scr, len(question_wrap) + 3, self.color['grey-dark'])
 
             # Set the offset to the next line
             selection_offset = len(question_wrap) + 3
@@ -542,7 +588,7 @@ class ExamTerminal:
             term_height, term_width = scr.getmaxyx()
 
             # Progress bar and status - call method
-            progress_bar = self.__get_progress_bar(exam_progress=self.questions_progress, bar_char_width=term_width - 23)
+            progress_bar = utlity.get_progress_bar(exam_progress=self.questions_progress, bar_char_width=term_width - 23)
             scr.addstr(term_height - 3, 3, f"[ {self.questions_complete:3.0f}  / {self.questions_total:3.0f}  ][{progress_bar}]", self.color['default'])
 
             # Elapsed Time
@@ -552,7 +598,7 @@ class ExamTerminal:
                 color = self.color['orange']
             if examp_elapsed_dec > 0.92:
                 color = self.color['red'] | self.decor['bold']
-            progress_bar = self.__get_progress_bar(exam_progress=examp_elapsed_dec, bar_char_width=term_width - 23)
+            progress_bar = utlity.get_progress_bar(exam_progress=examp_elapsed_dec, bar_char_width=term_width - 23)
             scr.addstr(term_height - 2, 3, f"[ {self.exam_elapsed_time:3.0f}s / {self.exam_allowed_time:3.0f}s ][{progress_bar}]", color)
 
             ########################################################################################
@@ -769,7 +815,7 @@ class ExamTerminal:
     def draw_result(self, scr) -> Tuple[str, bool]:
         # Setting up basic stuff for curses and load keys
         self.__basic_screen_setup(scr, halfdelay=False)
-        KEYS = self.__load_keys()
+        KEYS = utlity.load_keys()
 
         self.selection_index = 0
 
@@ -819,10 +865,10 @@ class ExamTerminal:
             self.__check_terminal_size(scr)
 
             # Drawing the screen border
-            self.__draw_screen_border(scr, self.color['grey-dark'])
+            utlity.draw_screen_border(scr, self.color['grey-dark'])
 
             # Show software name/title
-            scr.addstr(term_height - 2, 2, self.__load_software_ascii_name(), self.color['grey-dark'])
+            scr.addstr(term_height - 2, 2, utlity.load_software_ascii_name(), self.color['grey-dark'])
 
             ########################################################################################
 
@@ -830,10 +876,10 @@ class ExamTerminal:
 
             # Heading
             line = f"Exam Result Summary"
-            scr.addstr(start_y, self.__center_x(term_width, line), line, self.decor['bold'])
+            scr.addstr(start_y, utlity.center_x(term_width, line), line, self.decor['bold'])
             start_y += 2
 
-            self.__draw_horizontal_seperator(scr, start_y, self.color['grey-dark'])
+            utlity.draw_horizontal_seperator(scr, start_y, self.color['grey-dark'])
             start_y += 2
 
             # Draw items
@@ -841,13 +887,13 @@ class ExamTerminal:
             results = self.__assemble_exam_results()
             for index, item in results.items():
                 scr.addstr(start_y , start_x[0], item['label'], self.color['default'])
-                scr.addstr(start_y , start_x[1], self.__truncate_text(item['text'], term_width - 32), self.color[item['color']] | self.decor[item['decor']])
+                scr.addstr(start_y , start_x[1], utlity.truncate_text(item['text'], term_width - 32), self.color[item['color']] | self.decor[item['decor']])
                 start_y += item['skip_lines']
 
             ########################################################################################
 
             selections = ["Save Result PDF and Quit", "Main Menu", "Quit"]  # TODO: "Review Question"
-            self.__draw_horizontal_seperator(scr, term_height - len(selections) - 4, self.color['grey-dark'])
+            utlity.draw_horizontal_seperator(scr, term_height - len(selections) - 4, self.color['grey-dark'])
             start_y = term_height - len(selections) - 7
             self.__draw_selection_menu(scr, selections, start_y)
 
@@ -943,7 +989,7 @@ class ExamTerminal:
             else:
                 pdf.set_font('Helvetica', '', 11)
             pdf.set_xy(x=start_x[1], y=start_y)
-            pdf.cell(w=60, h=line_height, txt=self.__truncate_text(item['text'], self.width_limit - 25), border=0, align='L')
+            pdf.cell(w=60, h=line_height, txt=utlity.truncate_text(item['text'], self.width_limit - 25), border=0, align='L')
             
             start_y += item['skip_lines'] * 8
 
@@ -957,7 +1003,7 @@ class ExamTerminal:
         pdf.set_text_color(*[100, 100, 100])
         pdf.set_font('Helvetica', 'I', 8) 
         pdf.set_xy(x=page_left_margin + 3, y=page_height - page_bottom_margin - 8)
-        pdf.cell(w=0, h=5, txt=f"Created with {self.__load_software_ascii_name()}", border=0, align='L')
+        pdf.cell(w=0, h=5, txt=f"Created with {utlity.load_software_ascii_name()}", border=0, align='L')
 
         # Export the pdf to file
         datetime_text = datetime.fromtimestamp(self.exam_contents['exam']['exam_end_timestamp']).strftime("[%m-%d][%H-%M]")
@@ -1041,105 +1087,3 @@ class ExamTerminal:
 
         # Evaluate the exam
         self.__evaluate_exam()
-
-    ###############################################################################################
-
-    @staticmethod
-    def __load_curses_colors_decor() -> Tuple[dict, dict]:
-        # In code usage example:
-        #       scr.addstr(y, x, "hello", self.color['blue'])
-        #       scr.addstr(y, x, "hello", self.color['blue'] | self.decor['bold'])
-
-        # Start colors in curses
-        curses.start_color()
-
-        # Defining colors [foreground/font, background]
-        color_definition = {
-            'default':      [curses.COLOR_WHITE, 0],    # FIXME: Rename to "normal" to match decor
-            'red':          [curses.COLOR_RED, 0],
-            'green':        [curses.COLOR_GREEN, 0],
-            'blue':         [curses.COLOR_BLUE, 0],
-            'yellow':       [curses.COLOR_YELLOW, 0],
-            'orange':       [209, 0],
-            'cyan':         [curses.COLOR_CYAN, 0],
-            'magenta':      [curses.COLOR_MAGENTA, 0],
-            'grey-dark':    [240, 0],
-            'grey-light':   [248, 0],
-            'black-white':  [curses.COLOR_BLACK, curses.COLOR_WHITE],
-            'white-red':    [curses.COLOR_WHITE, curses.COLOR_RED]
-        }
-
-        # If terminal does not support colors, reset everything to white
-        if not curses.has_colors() or not curses.can_change_color():
-            for color in color_definition:
-                color_definition[color] = [curses.COLOR_WHITE, 0]
-
-        # Initiating curses color and saving for quick reference
-        color = {}
-        for index, (key, value) in enumerate(color_definition.items()):
-            try:
-                curses.init_pair(index + 1, value[0], value[1])
-            except:
-                curses.init_pair(index + 1, 0, 0)
-            color[key] = curses.color_pair(index + 1)
-
-        # Defining font decorations
-        decoration_definition ={
-            'normal' : curses.A_NORMAL,         # Normal display (no highlight)
-            'standout': curses.A_STANDOUT,      # Best highlighting mode of the terminal
-            'underline': curses.A_UNDERLINE,    # Underlining
-            'reverse': curses.A_REVERSE,        # Reverse video
-            'blink': curses.A_BLINK,            # Blinking
-            'dim': curses.A_DIM,                # Half bright
-            'bold': curses.A_BOLD,              # Extra bright or bold
-            'protect': curses.A_PROTECT,        # Protected mode
-            'invisible': curses.A_INVIS,        # Invisible or blank mode
-            'alt-char': curses.A_ALTCHARSET,    # Alternate character set
-            'char': curses.A_CHARTEXT           # Bit-mask to extract a character
-        }
-
-        # Initiating curses color and saving for quick reference
-        decor = {}
-        for key, value in decoration_definition.items():
-            decor[key] = value
-
-        return color, decor
-
-    @staticmethod
-    def __load_keys() -> dict:
-        KEYS = {
-            "ENTER":  (curses.KEY_ENTER, ord('\n'), ord('\r')),
-            "SPACE":  (32, ord(' ')),
-            "UP":     (curses.KEY_UP, ord('k')),
-            "DOWN":   (curses.KEY_DOWN, ord('j')),
-            "RIGHT":  (curses.KEY_RIGHT, ord('l')),
-            "LEFT":   (curses.KEY_LEFT, ord('h')),
-            "PAUSE":  (ord('p'), ord('P')),
-            "RESUME": (ord('r'), ord('R')),
-            "QUIT":   (27 , ord('q'), ord('Q'))
-        }
-        return KEYS
-
-    @staticmethod
-    def __load_software_ascii_name() -> str:
-        # TODO: Smarter, automatic loading of version
-        
-        software_name = "Exam Terminal"
-        software_version = "0.0.4"  # TODO: Pull from setup.py somehow
-        return software_name + ' v' + software_version
-
-    @staticmethod
-    def __center_x(display_width:int, line:str) -> int:
-        return display_width // 2 - len(line) // 2
-
-    @staticmethod
-    def __center_y(display_height:int) -> int:
-        return display_height // 2
-
-    @staticmethod
-    def __truncate_text(text:str, length:int) -> str:
-        truncated_text = text
-        if len(text) >= length - 3:
-            truncated_text = text[0:length - 3] + '...'
-
-        return truncated_text
